@@ -277,8 +277,7 @@ get_path_length_from_uri(const char *uri)
   if (!unescaped) return 0;
 
   ULOG_DEBUG("Original uri = %s", uri);
-  /* Should Not be utf8, but G_FILENAME_ENCODING, BUT it still is. */
-  len = g_utf8_strlen(unescaped, -1);
+  len = strlen (unescaped);
   ULOG_DEBUG("Unescaped path = %s, length = %d", unescaped, len);
   g_free(unescaped);
 
@@ -408,7 +407,7 @@ hildon_file_chooser_dialog_do_autonaming(HildonFileChooserDialogPrivate *
     g_assert(HILDON_IS_FILE_SELECTION(priv->filetree));
 
     if (GTK_WIDGET_VISIBLE(priv->caption_control_name) &&
-        priv->stub_name && !priv->edited)
+        priv->stub_name && priv->stub_name[0] && !priv->edited)
     {
         gchar *name = NULL;
         gboolean selection;
@@ -469,12 +468,10 @@ set_stub_and_ext (HildonFileChooserDialogPrivate *priv,
   char *dot;
   gboolean is_folder;
 
-  /* XXX - We do not always reset the extension here since the old
-           code didn't do it and some code out there might rely on it
-           not being done.
-  */
   g_free (priv->stub_name);
+  g_free (priv->ext_name);
   priv->stub_name = g_strdup (name);
+  priv->ext_name = NULL;
 
   /* XXX - Determine whether we are talking about a folder here.  If
            action is CREATE_FOLDER, the dialog might actually be used
@@ -647,6 +644,40 @@ static void hildon_file_chooser_dialog_current_folder_changed(GtkWidget *
     hildon_file_chooser_dialog_do_autonaming(priv);
 }
 
+/* If a row changes in the model, we check if the location label
+   should be updated */
+
+static void
+check_for_location_update (GtkTreeModel *model,
+			   GtkTreePath *path,
+			   GtkTreeIter *iter,
+			   gpointer data)
+{
+  HildonFileChooserDialogPrivate *priv = data;
+  GtkTreeIter current_iter;
+  GtkTreePath *current_path = NULL;	
+  
+  if (!priv)
+    return;
+  
+  if (!path)
+    return;
+  
+  if (!model)
+    return;
+  
+  if (priv->filetree && HILDON_IS_FILE_SELECTION (priv->filetree)) 
+    {
+      hildon_file_selection_get_current_folder_iter (priv->filetree,
+						     &current_iter);
+      current_path = gtk_tree_model_get_path (model, &current_iter);
+    }
+  
+  if (current_path
+      && gtk_tree_path_compare (path, current_path) == 0) 
+    hildon_file_chooser_dialog_update_location_info(priv);
+}
+
 static gboolean
 hildon_file_chooser_dialog_set_current_folder(GtkFileChooser * chooser,
                                               const GtkFilePath * path,
@@ -663,15 +694,19 @@ hildon_file_chooser_dialog_set_current_folder(GtkFileChooser * chooser,
 
     /* Now resplit the name into stub and ext parts since now the
        situation might have changed as to whether it is a folder or
-       not.
+       not.  Only do this with a non-empty stub, tho.
     */
-    if (self->priv->ext_name)
-      name = g_strconcat (self->priv->stub_name, self->priv->ext_name, NULL);
-    else
-      name = g_strdup (self->priv->stub_name);
-
-    set_stub_and_ext (self->priv, name);
-    g_free (name);
+    if (self->priv->stub_name && self->priv->stub_name[0])
+      {
+	if (self->priv->ext_name)
+	  name = g_strconcat (self->priv->stub_name,
+			      self->priv->ext_name, NULL);
+	else
+	  name = g_strdup (self->priv->stub_name);
+	
+	set_stub_and_ext (self->priv, name);
+	g_free (name);
+      }
 
     return result;
 }
@@ -704,6 +739,8 @@ static void hildon_file_chooser_dialog_set_current_name(GtkFileChooser *
     ULOG_INFO("Current name set: body = %s, ext = %s", priv->stub_name, priv->ext_name);
     hildon_file_chooser_dialog_set_limit(HILDON_FILE_CHOOSER_DIALOG(chooser));
     hildon_file_chooser_dialog_do_autonaming(priv);
+
+    set_entry (priv->entry_name, priv->stub_name, priv->ext_name);
 }
 
 static gboolean hildon_file_chooser_dialog_select_path(GtkFileChooser *
@@ -1134,6 +1171,7 @@ static GtkWidget
                                                        "file-system-model",
                                                        self->priv->model,
                                                        NULL);
+    gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
     sync_current_folders(self, HILDON_FILE_CHOOSER_DIALOG(dialog));
 
     return dialog;
@@ -1157,17 +1195,10 @@ static void create_folder_callback(GtkFileSystemHandle *handle,
     if (error) {
         if (g_error_matches(error, GTK_FILE_SYSTEM_ERROR,
 			    GTK_FILE_SYSTEM_ERROR_ALREADY_EXISTS))
-            message = _("ckdg_ib_folder_already_exists");
+            message = HCS("ckdg_ib_folder_already_exists");
         else
             message = HCS("sfil_ni_operation_failed");
 
-	fprintf (stderr, "domain %d (%d)\n",
-		 error->domain, GTK_FILE_SYSTEM_ERROR);
-	fprintf (stderr, "code %d (%d)\n",
-		 error->code, GTK_FILE_SYSTEM_ERROR_ALREADY_EXISTS);
-
-        fprintf (stderr, "%s\n", error->message);
-
         hildon_banner_show_information (GTK_WIDGET(self), NULL, message);
         hildon_file_chooser_dialog_select_text(self->priv);
         gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_OK, TRUE);
@@ -1491,6 +1522,12 @@ static void hildon_file_chooser_dialog_set_property(GObject * object,
       g_assert(HILDON_IS_FILE_SELECTION(priv->filetree));
       hildon_file_selection_set_mode(priv->filetree,
                                      g_value_get_enum (value));
+      if (g_value_get_enum (value) == HILDON_FILE_SELECTION_MODE_LIST)
+    	gtk_check_menu_item_set_active
+	  (GTK_CHECK_MENU_ITEM(priv->mode_list), TRUE);
+      else
+    	gtk_check_menu_item_set_active
+	  (GTK_CHECK_MENU_ITEM(priv->mode_thumbnails), TRUE);
       break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -2036,6 +2073,8 @@ hildon_file_chooser_dialog_constructor(GType type,
     g_signal_connect_object(priv->model, "finished-loading",
                      G_CALLBACK(hildon_file_chooser_dialog_finished_loading),
                      obj, 0);
+    g_signal_connect (G_OBJECT (priv->model), "row-changed", 
+		      G_CALLBACK (check_for_location_update), priv);
 
     gtk_dialog_set_has_separator(GTK_DIALOG(obj), FALSE);
     hildon_file_chooser_dialog_set_limit(HILDON_FILE_CHOOSER_DIALOG(obj));
