@@ -31,6 +31,12 @@
 #include "hildon-file-common-private.h"
 #include "hildon-file-system-private.h"
 
+#define GCONF_PATH "/system/osso/af"
+#define USED_OVER_USB_KEY GCONF_PATH "/mmc-used-over-usb"
+#define USED_OVER_USB_INTERNAL_KEY GCONF_PATH "/internal-mmc-used-over-usb"
+#define CORRUPTED_MMC_KEY GCONF_PATH "/mmc-corrupted"
+#define CORRUPTED_INTERNAL_MMC_KEY GCONF_PATH "/internal-mmc-corrupted"
+
 static void
 hildon_file_system_voldev_class_init (HildonFileSystemVoldevClass *klass);
 static void
@@ -39,7 +45,8 @@ static void
 hildon_file_system_voldev_init (HildonFileSystemVoldev *device);
 
 static gboolean
-hildon_file_system_voldev_is_visible (HildonFileSystemSpecialLocation *location);
+hildon_file_system_voldev_is_visible (HildonFileSystemSpecialLocation *location,
+				      gboolean has_children);
 
 static void
 hildon_file_system_voldev_volumes_changed (HildonFileSystemSpecialLocation
@@ -48,17 +55,54 @@ hildon_file_system_voldev_volumes_changed (HildonFileSystemSpecialLocation
 static char *
 hildon_file_system_voldev_get_extra_info (HildonFileSystemSpecialLocation
 					  *location);
+static void init_vol_type (const char *path,
+                           HildonFileSystemVoldev *voldev);
 
 G_DEFINE_TYPE (HildonFileSystemVoldev,
                hildon_file_system_voldev,
                HILDON_TYPE_FILE_SYSTEM_SPECIAL_LOCATION);
 
+enum {
+    PROP_USED_OVER_USB = 1
+};
+
+static void
+gconf_value_changed(GConfClient *client, guint cnxn_id,
+                    GConfEntry *entry, gpointer data)
+{
+    HildonFileSystemVoldev *voldev;
+    HildonFileSystemSpecialLocation *location;
+    gboolean change = FALSE;
+
+    voldev = HILDON_FILE_SYSTEM_VOLDEV (data);
+    location = HILDON_FILE_SYSTEM_SPECIAL_LOCATION (voldev);
+
+    if (!voldev->vol_type_valid)
+      init_vol_type (location->basepath, voldev);
+
+    if (voldev->vol_type == INT_CARD &&
+        g_ascii_strcasecmp (entry->key, USED_OVER_USB_INTERNAL_KEY) == 0)
+      change = TRUE;
+    else if (voldev->vol_type == EXT_CARD &&
+             g_ascii_strcasecmp (entry->key, USED_OVER_USB_KEY) == 0)
+      change = TRUE;
+
+    if (change)
+      {
+        voldev->used_over_usb = gconf_value_get_bool (entry->value);
+        ULOG_DEBUG_F("%s = %d", entry->key, voldev->used_over_usb);
+        g_signal_emit_by_name (location, "changed");
+        g_signal_emit_by_name (data, "rescan");
+      }
+}
+
 static void
 hildon_file_system_voldev_class_init (HildonFileSystemVoldevClass *klass)
 {
     GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
     HildonFileSystemSpecialLocationClass *location =
             HILDON_FILE_SYSTEM_SPECIAL_LOCATION_CLASS (klass);
+    GError *error = NULL;
 
     gobject_class->finalize = hildon_file_system_voldev_finalize;
 
@@ -66,16 +110,36 @@ hildon_file_system_voldev_class_init (HildonFileSystemVoldevClass *klass)
     location->is_visible = hildon_file_system_voldev_is_visible;
     location->volumes_changed = hildon_file_system_voldev_volumes_changed;
     location->get_extra_info = hildon_file_system_voldev_get_extra_info;
+
+    klass->gconf = gconf_client_get_default ();
+    gconf_client_add_dir (klass->gconf, GCONF_PATH,
+                          GCONF_CLIENT_PRELOAD_NONE, &error);
+    if (error != NULL)
+      {
+        ULOG_ERR_F ("gconf_client_add_dir failed: %s", error->message);
+        g_error_free (error);
+      }
 }
 
 static void
 hildon_file_system_voldev_init (HildonFileSystemVoldev *device)
 {
     HildonFileSystemSpecialLocation *location;
+    GError *error = NULL;
+    HildonFileSystemVoldevClass *klass =
+            HILDON_FILE_SYSTEM_VOLDEV_GET_CLASS (device);
 
     location = HILDON_FILE_SYSTEM_SPECIAL_LOCATION (device);
     location->compatibility_type = HILDON_FILE_SYSTEM_MODEL_MMC;
     location->failed_access_message = NULL;
+
+    gconf_client_notify_add (klass->gconf, GCONF_PATH,
+                             gconf_value_changed, device, NULL, &error);
+    if (error != NULL)
+      {
+        ULOG_ERR_F ("gconf_client_notify_add failed: %s", error->message);
+        g_error_free (error);
+      }
 }
 
 static void
@@ -94,16 +158,51 @@ hildon_file_system_voldev_finalize (GObject *obj)
 }
 
 static gboolean
-hildon_file_system_voldev_is_visible (HildonFileSystemSpecialLocation *location)
+hildon_file_system_voldev_is_visible (HildonFileSystemSpecialLocation *location,
+				      gboolean has_children)
 {
   HildonFileSystemVoldev *voldev = HILDON_FILE_SYSTEM_VOLDEV (location);
+  HildonFileSystemVoldevClass *klass =
+          HILDON_FILE_SYSTEM_VOLDEV_GET_CLASS (voldev);
   gboolean visible = FALSE;
+  GError *error = NULL;
+  gboolean value, corrupted;
 
-  if (voldev->volume)
+  if (!voldev->vol_type_valid)
+    init_vol_type (location->basepath, voldev);
+
+  if (voldev->vol_type == INT_CARD) {
+    value = gconf_client_get_bool (klass->gconf,
+                                   USED_OVER_USB_INTERNAL_KEY, &error);
+    corrupted = gconf_client_get_bool (klass->gconf,
+				       CORRUPTED_INTERNAL_MMC_KEY, &error);
+  } else if (voldev->vol_type == EXT_CARD) {
+    value = gconf_client_get_bool (klass->gconf,
+                                   USED_OVER_USB_KEY, &error);
+    corrupted = gconf_client_get_bool (klass->gconf,
+				       CORRUPTED_MMC_KEY, &error);
+  } else
+    value = corrupted = FALSE; /* USB_STORAGE */
+
+  if (error)
+    {
+      ULOG_ERR_F ("gconf_client_get_bool failed: %s", error->message);
+      g_error_free (error);
+    }
+  else
+    voldev->used_over_usb = value;
+
+  ULOG_DEBUG_F("%s type: %d, used_over_usb: %d", location->basepath,
+               voldev->vol_type, voldev->used_over_usb);
+
+  if (voldev->volume && !voldev->used_over_usb)
     visible = gnome_vfs_volume_is_mounted (voldev->volume);
-  else if (voldev->drive)
+  else if (voldev->drive && voldev->vol_type == USB_STORAGE)
+    visible = FALSE; /* USB drives are never visible */
+  else if (voldev->drive && !voldev->used_over_usb)
     visible = (gnome_vfs_drive_is_connected (voldev->drive)
-               && !gnome_vfs_drive_is_mounted (voldev->drive));
+	       && !gnome_vfs_drive_is_mounted (voldev->drive)
+	       && corrupted);
 
   return visible;
 }
@@ -211,6 +310,71 @@ beautify_mmc_name (char *name, gboolean internal)
   return name;
 }
 
+static void init_vol_type (const char *path,
+                           HildonFileSystemVoldev *voldev)
+{
+  HildonFileSystemVoldevClass *klass;
+  gchar *value;
+  gboolean drive;
+
+  if (voldev->vol_type_valid)
+    /* already initialised */
+    return;
+
+  if (path == NULL)
+    {
+      ULOG_WARN_F ("path == NULL");
+      return;
+    }
+
+  klass = HILDON_FILE_SYSTEM_VOLDEV_GET_CLASS (voldev);
+
+  if (g_str_has_prefix (path, "drive:///dev/sd") ||
+      g_str_has_prefix (path, "file:///media/usb/"))
+    {
+      voldev->vol_type = USB_STORAGE;
+      voldev->vol_type_valid = TRUE;
+      return;
+    }
+  else if (g_str_has_prefix (path, "drive://"))
+    {
+      drive = TRUE;
+      value = gconf_client_get_string (klass->gconf,
+                    "/system/osso/af/mmc-device-name", NULL);
+    }
+  else
+    {
+      drive = FALSE;
+      value = gconf_client_get_string (klass->gconf,
+                    "/system/osso/af/mmc-mount-point", NULL);
+    }
+
+  if (value)
+    {
+      char buf[100];
+
+      if (drive)
+        {
+          snprintf (buf, 100, "drive://%s", value);
+          if (g_str_has_prefix (path, buf))
+            voldev->vol_type = EXT_CARD;
+          else
+            voldev->vol_type = INT_CARD;
+        }
+      else
+        {
+          snprintf (buf, 100, "file://%s", value);
+          if (strncmp (buf, path, 100) == 0)
+            voldev->vol_type = EXT_CARD;
+          else
+            voldev->vol_type = INT_CARD;
+        }
+
+      voldev->vol_type_valid = TRUE;
+      g_free (value);
+    }
+}
+
 static void
 hildon_file_system_voldev_volumes_changed (HildonFileSystemSpecialLocation
                                            *location, GtkFileSystem *fs)
@@ -233,6 +397,9 @@ hildon_file_system_voldev_volumes_changed (HildonFileSystemSpecialLocation
   else
     voldev->volume = find_volume (location->basepath);
 
+  if (!voldev->vol_type_valid)
+    init_vol_type (location->basepath, voldev);
+
   if (voldev->volume)
     {
       location->fixed_title =
@@ -248,30 +415,30 @@ hildon_file_system_voldev_volumes_changed (HildonFileSystemSpecialLocation
   /* XXX - GnomeVFS should provide the right icons and display names.
    */
 
+  location->sort_weight = SORT_WEIGHT_USB;
   if (location->fixed_icon)
     {
       if (strcmp (location->fixed_icon, "gnome-dev-removable-usb") == 0
 	  || strcmp (location->fixed_icon, "gnome-dev-harddisk-usb") == 0)
-	location->fixed_icon = "qgn_list_filesys_removable_storage";
+	location->fixed_icon = g_strdup ("qgn_list_filesys_removable_storage");
       else if (strcmp (location->fixed_icon, "gnome-dev-removable") == 0
 	       || strcmp (location->fixed_icon, "gnome-dev-media-sdmmc") == 0)
 	{
-	  gboolean internal = 
-	    g_str_has_prefix (location->basepath, "file:///media/mmc2");
-	  
-	  /* the following strcmp() is for drives, so that we don't need
-	   * knowledge about device naming */
-	  if (strcmp(location->fixed_title,
-		     "mmc-undefined-name-internal") == 0)
-	    internal = TRUE;
-
-	  if (internal)
-	    location->fixed_icon = "qgn_list_gene_internal_memory_card";
+	  if (voldev->vol_type == INT_CARD)
+	    {
+	      location->sort_weight = SORT_WEIGHT_INTERNAL_MMC;
+	      location->fixed_icon =
+		g_strdup ("qgn_list_gene_internal_memory_card");
+	    }
 	  else
-	    location->fixed_icon = "qgn_list_gene_removable_memory_card";
+	    {
+	      location->sort_weight = SORT_WEIGHT_EXTERNAL_MMC;
+	      location->fixed_icon = 
+		g_strdup ("qgn_list_gene_removable_memory_card");
+	    }
 	  
 	  location->fixed_title = beautify_mmc_name (location->fixed_title,
-						     internal);
+                                      voldev->vol_type == INT_CARD);
 	}
     }
 
